"""v0.9.0."""
import sqlalchemy as sa  # noqa
import sqlmodel
from alembic import op

revision = "4429de4e490b"
down_revision = "1df05e5b531f"


def upgrade() -> None:
    bind = op.get_bind()
    sqlite = bind.engine.name == "sqlite"

    if sqlite:
        prefix, schema = "bionty.", None
        idx = "ix_bionty."
        pkc = "pk_bionty."
    else:
        prefix, schema = "", "bionty"
        idx = "ix_bionty_"
        pkc = "pk_bionty_"

    op.create_table(
        f"{prefix}cell_line",
        sa.Column("id", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("ontology_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.PrimaryKeyConstraint("id", name=op.f(f"{pkc}cell_line")),
        schema=schema,
    )
    op.create_index(
        op.f(f"{idx}cell_line_name"),
        f"{prefix}cell_line",
        ["name"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f(f"{idx}cell_line_ontology_id"),
        f"{prefix}cell_line",
        ["ontology_id"],
        unique=True,
        schema=schema,
    )

    op.create_table(
        f"{prefix}pathway",
        sa.Column("id", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("ontology_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.PrimaryKeyConstraint("id", name=op.f(f"{pkc}pathway")),
        schema=schema,
    )
    op.create_index(
        op.f(f"{idx}pathway_name"),
        f"{prefix}pathway",
        ["name"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f(f"{idx}pathway_ontology_id"),
        f"{prefix}pathway",
        ["ontology_id"],
        unique=True,
        schema=schema,
    )

    op.create_table(
        f"{prefix}phenotype",
        sa.Column("id", sqlmodel.sql.sqltypes.AutoString(), nullable=False),
        sa.Column("ontology_id", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.Column("name", sqlmodel.sql.sqltypes.AutoString(), nullable=True),
        sa.PrimaryKeyConstraint("id", name=op.f(f"{pkc}phenotype")),
        schema=schema,
    )
    op.create_index(
        op.f(f"{idx}phenotype_name"),
        f"{prefix}phenotype",
        ["name"],
        unique=False,
        schema=schema,
    )
    op.create_index(
        op.f(f"{idx}phenotype_ontology_id"),
        f"{prefix}phenotype",
        ["ontology_id"],
        unique=True,
        schema=schema,
    )


def downgrade() -> None:
    pass



from pathlib import Path
from typing import Union

import requests  # type:ignore
import yaml  # type:ignore
from lndb.dev.upath import UPath
from rich import print
from rich.progress import Progress

from bionty._settings import settings


def load_yaml(
    filename: Union[str, Path], convert_dates: bool = True
):  # pragma: no cover
    with open(filename, "r") as f:
        return yaml.safe_load(f)


def write_yaml(
    data: dict,
    filename: Union[str, Path],
    sort_keys: bool = False,
    default_flow_style: bool = False,
):  # pragma: no cover
    with open(filename, "w") as f:
        yaml.dump(
            data,
            f,
            sort_keys=sort_keys,
            default_flow_style=default_flow_style,
        )


def url_download(  # pragma: no cover
    url: str, filename: Union[str, Path, None] = None, block_size: int = 1024, **kwargs
) -> None:
    """Downloads a file to a specified path.

    Args:
        url: The URL to download.
        filename: The path to download the file to.
        block_size: Buffer size in bytes for sending a file-like message body.
        **kwargs: Keyword arguments are passed to 'requests'

    Raises:
        HttpError: If the request response is not 200 and OK.
    """
    try:
        response = requests.get(url, stream=True, allow_redirects=True, **kwargs)
        response.raise_for_status()

        total_content_length = int(response.headers.get("content-length", 0))
        if filename is None:
            filename = url.split("/")[-1]

        with Progress(refresh_per_second=10000) as progress:
            task = progress.add_task("[red]Downloading...", total=total_content_length)

            with open(filename, "wb") as file:
                for data in response.iter_content(block_size):
                    file.write(data)
                    progress.update(task, advance=block_size)

    except requests.exceptions.HTTPError as err:
        print(err)


def s3_bionty_assets(
    filename: str, localpath: Path = None, assets_base_url: str = "s3://bionty-assets"
):
    """Synchronizes a S3 file path with local file storage.

    If the file does not exist locally it gets downloaded.
    If the file does not exist on S3, the file does not get synchronized.

    Args:
        filename: The suffix of the assets_base_url.
        localpath: The Path to the local file.
        assets_base_url: The S3 base URL. Prefix of the filename.

    Returns:
        A Path object of the synchronized path.
    """
    cloudpath = UPath(f"{assets_base_url}/{filename}", anon=True, cache_regions=True)
    if not localpath:
        localpath = settings.datasetdir / filename

    cloudpath.synchronize(localpath)

    return localpath



import os
from pathlib import Path
from typing import Optional

import pandas as pd
from rich.console import Console
from rich.table import Table

from bionty._settings import settings
from bionty.dev._io import load_yaml

console = Console()

ROOT_DIR = Path(__file__).parent.resolve()


def display_available_versions(
    return_df: bool = False,
) -> Optional[pd.DataFrame]:  # pragma: no cover
    """Displays all available entities and versions in a Rich table.

    Args:
        return_df: Whether to return a Pandas DataFrame containing the available versions. Defaults to False.
    """
    VERSIONS_FILE_PATH = (settings.versionsdir / "local.yaml").resolve()
    versions = load_yaml(VERSIONS_FILE_PATH)

    table = Table(title="Available versions")

    table.add_column("Ontology", justify="right", style="cyan", no_wrap=True)
    table.add_column("URL", justify="right", style="cyan", no_wrap=True)
    table.add_column("Bionty Entity", justify="right", style="cyan", no_wrap=True)
    table.add_column("Database key", justify="right", style="cyan", no_wrap=True)
    table.add_column("All versions", justify="right", style="cyan", no_wrap=True)

    df_rows = []
    for entity, db_to_version in versions.items():
        if entity == "version":
            continue
        for db, _to_versions_url in db_to_version.items():
            versions = ""
            _ontology_name = _to_versions_url["name"]
            _ontology_url = _to_versions_url["website"]
            for version_str, url_md5 in _to_versions_url["versions"].items():
                versions += str(version_str) + "\n"

                # Compatibility code for old local.yml files that may not yet have md5s
                if len(url_md5) > 1:
                    url = url_md5[0]
                else:
                    url = url_md5

                df_rows.append(
                    {
                        "Ontology": _ontology_name,
                        "URL": url,
                        "Bionty Entity": entity,
                        "Database key": db,
                        "All versions": str(version_str),
                    }
                )
                table.add_row(_ontology_name, _ontology_url, entity, db, versions)

    if return_df:
        df = pd.DataFrame(df_rows)
        return df

    console.print(table)
    return None


def display_active_versions(
    return_df: bool = False,
) -> Optional[pd.DataFrame]:  # pragma: no cover
    """Displays all currently set as default entities and versions in a Rich table.

    Args:
        return_df: Whether to return a Pandas DataFrame containing the available versions. Defaults to False.
    """
    version_table = (
        "._lndb.yaml" if os.getenv("LAMINDB_INSTANCE_LOADED") == 1 else "._current.yaml"
    )

    VERSIONS_FILE_PATH = Path(f"{ROOT_DIR}/versions/{version_table}").resolve()
    versions = load_yaml(VERSIONS_FILE_PATH.resolve())

    table = Table(title=f"Currently used versions in {version_table}")

    table.add_column("Entity", justify="right", style="cyan", no_wrap=True)
    table.add_column("Database", justify="right", style="cyan", no_wrap=True)
    table.add_column("Version", justify="right", style="cyan", no_wrap=True)

    df_rows = []
    for entity, db_to_version in versions.items():
        for db, version in db_to_version.items():
            df_rows.append({"Entity": entity, "Database": db, "Version": str(version)})
            table.add_row(entity, db, str(version))

    if return_df:
        df = pd.DataFrame(df_rows)
        return df

    console.print(table)
    return None



"""v0.10.0."""
import sqlalchemy as sa  # noqa
import sqlmodel  # noqa
from alembic import op

revision = "5594b330a854"
down_revision = "4429de4e490b"


def upgrade() -> None:
    bind = op.get_bind()
    sqlite = bind.engine.name == "sqlite"

    if sqlite:
        prefix, schema = "bionty.", None
    else:
        prefix, schema = "", "bionty"

    op.alter_column(
        table_name=f"{prefix}cell_marker",
        column_name="ncbi_gene_ids",
        new_column_name="ncbi_gene_id",
        schema=schema,
    )

    op.alter_column(
        table_name=f"{prefix}cell_marker",
        column_name="gene_symbols",
        new_column_name="gene_symbol",
        schema=schema,
    )

    op.alter_column(
        table_name=f"{prefix}cell_marker",
        column_name="protein_names",
        new_column_name="gene_name",
        schema=schema,
    )

    op.alter_column(
        table_name=f"{prefix}cell_marker",
        column_name="uniprotkb_ids",
        new_column_name="uniprotkb_id",
        schema=schema,
    )


def downgrade() -> None:
    pass



import sys
from typing import Optional

import bionty as bt
from lamin_logger import logger

from ..dev import id

config_bionty_species = "human"


# https://stackoverflow.com/questions/128573/using-property-on-classmethods/64738850#64738850
class classproperty(object):
    def __init__(self, fget):
        self.fget = fget

    def __get__(self, owner_self, owner_cls):
        """Get."""
        if "sphinx" not in sys.modules:
            return self.fget(owner_cls)
        else:
            return "sphinx-build: property"


def fields_from_knowledge(
    locals: dict,
    entity: bt.Entity,
):
    kwargs = {}
    for k, v in locals.items():
        df = entity.df().reset_index().set_index(k)
        if v not in df.index:
            continue
        kwargs = df.loc[v].to_dict()
        kwargs[k] = v
        if "ontology_id" in kwargs:
            # TODO: save to Readout.df() like others
            if entity._entity == "readout":
                kwargs = entity.get(kwargs["ontology_id"])
            kwargs["id"] = kwargs["ontology_id"]
    pydantic_attrs = kwargs
    return pydantic_attrs


def set_attributes(model, pydantic_attrs):
    if len(pydantic_attrs) == 0:
        return

    for k, v in pydantic_attrs.items():
        if k not in model.__fields__:
            continue
        model.__setattr__(k, v)


def config_bionty(species: Optional[str] = None):
    global config_bionty_species
    if species is not None:
        config_bionty_species_ = config_bionty_species
        config_bionty_species = species
        if species != config_bionty_species_:
            logger.info(f"Configured species as {species}.")


def knowledge(sqlmodel_class):
    name = sqlmodel_class.__name__
    Entity = getattr(bt, name)

    def init_entity():
        try:
            return Entity(species=config_bionty_species)
        except TypeError:
            # For the Species entity
            return Entity()

    @classproperty
    def bionty(cls):
        return init_entity()

    @classmethod
    def from_bionty(cls, lookup_result: Optional[tuple] = None, **kwargs):
        if isinstance(lookup_result, tuple) and lookup_result is not None:
            return sqlmodel_class(lookup_result=lookup_result)
        else:
            pydantic_attrs = fields_from_knowledge(locals=kwargs, entity=sqlmodel_class.bionty)
            if len(pydantic_attrs) == 0:
                raise ValueError(
                    "No entry is found in bionty reference table with kwargs!\nPlease"
                    " check you configured the correct species with"
                    " `.config_bionty(species=...)`"
                )
            pydantic_attrs = _encode_id(pydantic_attrs)
            if "species_id" not in pydantic_attrs and "species" not in pydantic_attrs:
                pydantic_attrs = _add_species(pydantic_attrs)
            return sqlmodel_class(**pydantic_attrs)

    def _add_species(pydantic_attrs: dict):
        if "species_id" in sqlmodel_class.__fields__:
            import lamindb as ln

            from .._core import Species

            sp = ln.select(Species, name=config_bionty_species).one_or_none()
            if sp is None:
                sp = Species.from_bionty(name=config_bionty_species)
            pydantic_attrs["species"] = sp

        return pydantic_attrs

    def _encode_id(pydantic_attrs: dict):
        if "id" in pydantic_attrs:
            id_encoder = getattr(id, sqlmodel_class.__table__.name.split(".")[-1])
            pydantic_attrs["id"] = id_encoder(pydantic_attrs["id"])
        return pydantic_attrs

    orig_init = sqlmodel_class.__init__

    def __init__(self, lookup_result: Optional[tuple] = None, **kwargs):
        if isinstance(lookup_result, tuple) and lookup_result is not None:
            kwargs = lookup_result._asdict()  # type:ignore
            kwargs = _encode_id(kwargs)
            if "species_id" not in kwargs and "species" not in kwargs:
                kwargs = _add_species(kwargs)

        orig_init(self, **kwargs)

    # def __init__(
    #     self,
    #     database: Optional[str] = None,
    #     version: Optional[str] = None,
    #     species: Optional[str] = None,
    #     knowledge_coupling: Optional[bool] = None,
    #     **kwargs,
    # ):
    #     orig_init(self, **kwargs)

    #     if knowledge_coupling is None:
    #         if name in features_entities:
    #             knowledge_coupling = False
    #         else:
    #             knowledge_coupling = True

    #     if knowledge_coupling:
    #         pydantic_attrs = fields_from_knowledge(locals=kwargs, entity=entity)
    #         set_attributes(self, pydantic_attrs)

    # def __new__(
    #     cls,
    #     database: Optional[str] = None,
    #     version: Optional[str] = None,
    #     species: Optional[str] = None,
    #     knowledge_coupling: bool = True,
    #     **kwargs,
    # ):
    #     entity_kwargs = {
    #         k: v
    #         for k, v in locals().items()
    #         if v is not None and k in ["database", "version", "species"]
    #     }
    #     if database is not None or species is not None:
    #         return Entity(**entity_kwargs)
    #     else:
    #         return orig_new(cls)

    # def __call__(cls, knowledge_coupling=True, **kwargs):
    #     return sqlmodel_class(knowledge_coupling=knowledge_coupling, **kwargs)

    sqlmodel_class.__init__ = __init__
    # sqlmodel_class.__new__ = __new__
    # Entity.__call__ = __call__
    sqlmodel_class.from_bionty = from_bionty
    sqlmodel_class.config_bionty = config_bionty
    sqlmodel_class.bionty = bionty

    return sqlmodel_class



from typing import Literal, Optional

from .._entity import Entity
from ._shared_docstrings import _doc_params, doc_entites


@_doc_params(doc_entities=doc_entites)
class CellLine(Entity):
    """Cell line.

    1. Cell Line Ontology
    Edits of terms are coordinated and reviewed on:
    https://github.com/CLO-ontology/CLO

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        species: str = "human",
        source: Optional[Literal["clo"]] = None,
        version: Optional[str] = None,
        **kwargs
    ) -> None:
        super().__init__(source=source, version=version, species=species, **kwargs)



"""v0.5.0.

Revision ID: 98da12fc80a8
Revises: 267d12e6f6f1
Create Date: 2022-10-31
"""
import sqlalchemy as sa  # noqa
import sqlmodel  # noqa
from alembic import op

revision = "afda12fc80a8"
down_revision = "267d12e6f6f1"
branch_labels = None
depends_on = None


def upgrade() -> None:
    bind = op.get_bind()
    if bind.engine.name == "sqlite":
        op.rename_table(old_table_name="gene", new_table_name="bionty.gene")
        op.rename_table(old_table_name="protein", new_table_name="bionty.protein")
        op.rename_table(old_table_name="cell_marker", new_table_name="bionty.cell_marker")
        op.rename_table(old_table_name="cell_type", new_table_name="bionty.cell_type")
        op.rename_table(old_table_name="species", new_table_name="bionty.species")
        op.rename_table(old_table_name="tissue", new_table_name="bionty.tissue")
        op.rename_table(old_table_name="disease", new_table_name="bionty.disease")
        op.rename_table(old_table_name="featureset", new_table_name="bionty.featureset")
        op.rename_table(old_table_name="featureset_gene", new_table_name="bionty.featureset_gene")
        op.rename_table(
            old_table_name="featureset_protein",
            new_table_name="bionty.featureset_protein",
        )
        op.rename_table(
            old_table_name="featureset_cell_marker",
            new_table_name="bionty.featureset_cell_marker",
        )
    else:
        op.execute("alter table public.gene set schema bionty")
        op.execute("alter table public.protein set schema bionty")
        op.execute("alter table public.cell_marker set schema bionty")
        op.execute("alter table public.cell_type set schema bionty")
        op.execute("alter table public.species set schema bionty")
        op.execute("alter table public.tissue set schema bionty")
        op.execute("alter table public.disease set schema bionty")
        op.execute("alter table public.featureset set schema bionty")
        op.execute("alter table public.featureset_gene set schema bionty")
        op.execute("alter table public.featureset_protein set schema bionty")
        op.execute("alter table public.featureset_cell_marker set schema bionty")


def downgrade() -> None:
    pass



"""Biological entities (`zdno`).

Import the package::

   import lnschema_bionty

Biological entities, all initialized via `Bionty <https://lamin.ai/docs/bionty>`__:

.. autosummary::
   :toctree: .

   Gene
   Protein
   CellMarker
   CellType
   CellLine
   Species
   Tissue
   Disease
   Pathway
   Phenotype
   Readout

Development tools:

.. autosummary::
   :toctree: .

   dev
   link

"""
# This is lnschema-module zdno.
_schema_id = "zdno"
_name = "bionty"
_migration = "3bd9e094a433"
__version__ = "0.16.3"  # Denote a release candidate of version 0.1.0 with 0.1rc1

from . import dev, link  # noqa
from ._core import (  # noqa
    CellLine,
    CellMarker,
    CellType,
    Disease,
    Gene,
    Pathway,
    Phenotype,
    Protein,
    Readout,
    Species,
    Tissue,
)
from .dev import id  # backward compat



from datetime import datetime as datetime
from typing import Optional

from lnschema_core import Features, File
from lnschema_core._timestamps import CreatedAt, UpdatedAt
from lnschema_core._users import CreatedBy
from lnschema_core.dev.sqlmodel import schema_sqlmodel
from sqlalchemy.orm import relationship
from sqlmodel import Field, Relationship

from . import _name as schema_name
from ._link import FeaturesCellMarker, FeaturesGene, FeaturesProtein, FileReadout
from .dev import id as idg
from .dev._bionty import knowledge

SQLModel, prefix, schema_arg = schema_sqlmodel(schema_name)


@knowledge
class Species(SQLModel, table=True):  # type: ignore
    """Species."""

    id: str = Field(default_factory=idg.species, primary_key=True)
    name: str = Field(default=None, index=True, unique=True)
    taxon_id: int = Field(default=None, index=True, unique=True)
    scientific_name: str = Field(default=None, index=True, unique=True)


@knowledge
class Gene(SQLModel, table=True):  # type: ignore
    """Genes."""

    id: str = Field(default_factory=idg.gene, primary_key=True)
    ensembl_gene_id: Optional[str] = Field(default=None, index=True)
    symbol: Optional[str] = Field(default=None, index=True)
    gene_type: Optional[str] = Field(default=None, index=True)
    description: Optional[str] = None
    ncbi_gene_id: int = Field(default=None, index=True)
    hgnc_id: Optional[str] = Field(default=None, index=True)
    mgi_id: Optional[str] = Field(default=None, index=True)
    omim_id: Optional[int] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    species_id: Optional[str] = Field(default=None, foreign_key="bionty.species.id", index=True)
    species: Species = Relationship()
    version: Optional[str] = None
    features: Features = Relationship(
        back_populates="genes",
        sa_relationship_kwargs=dict(secondary=FeaturesGene.__table__),
    )


Features.genes = relationship(Gene, back_populates="features", secondary=FeaturesGene.__table__)


@knowledge
class Protein(SQLModel, table=True):  # type: ignore
    """Proteins."""

    id: str = Field(default_factory=idg.protein, primary_key=True)
    name: str = Field(default=None, index=True)
    uniprotkb_id: str = Field(default=None, index=True)
    uniprotkb_name: str = Field(default=None, index=True)
    protein_names: Optional[str] = Field(default=None, index=True)
    length: Optional[int] = None
    species_id: str = Field(default=None, foreign_key="bionty.species.id")
    species: Species = Relationship()
    gene_symbols: Optional[str] = None
    gene_synonyms: Optional[str] = None
    ensembl_transcript_ids: Optional[str] = Field(default=None, index=True)
    ncbi_gene_ids: Optional[str] = Field(default=None, index=True)
    features: Features = Relationship(
        back_populates="proteins",
        sa_relationship_kwargs=dict(secondary=FeaturesProtein.__table__),
    )


Features.proteins = relationship(Protein, back_populates="features", secondary=FeaturesProtein.__table__)


@knowledge
class CellMarker(SQLModel, table=True):  # type: ignore
    """Cell markers."""

    __tablename__ = f"{prefix}cell_marker"

    id: str = Field(default_factory=idg.cell_marker, primary_key=True)
    name: str = Field(default=None, index=True, unique=True)
    ncbi_gene_id: Optional[str] = Field(default=None, index=True)
    gene_symbol: Optional[str] = Field(default=None, index=True)
    gene_name: Optional[str] = Field(default=None, index=True)
    uniprotkb_id: Optional[str] = Field(default=None, index=True)
    synonyms: Optional[str] = Field(default=None, index=True)
    species_id: str = Field(default=None, foreign_key="bionty.species.id")
    species: Species = Relationship()
    features: Features = Relationship(
        back_populates="cell_markers",
        sa_relationship_kwargs=dict(secondary=FeaturesCellMarker.__table__),
    )


Features.cell_markers = relationship(CellMarker, back_populates="features", secondary=FeaturesCellMarker.__table__)


@knowledge
class Tissue(SQLModel, table=True):  # type: ignore
    """Tissues."""

    id: str = Field(default_factory=idg.tissue, primary_key=True)
    ontology_id: str = Field(default=None, index=True, unique=True)
    name: str = Field(default=None, index=True)


@knowledge
class CellType(SQLModel, table=True):  # type: ignore
    """Cell types."""

    __tablename__ = f"{prefix}cell_type"

    id: str = Field(default_factory=idg.cell_type, primary_key=True)
    ontology_id: str = Field(default=None, index=True, unique=True)
    name: str = Field(default=None, index=True)


@knowledge
class Disease(SQLModel, table=True):  # type: ignore
    """Diseases."""

    id: str = Field(default_factory=idg.disease, primary_key=True)
    ontology_id: str = Field(default=None, index=True, unique=True)
    name: str = Field(default=None, index=True)


@knowledge
class CellLine(SQLModel, table=True):  # type: ignore
    """Cell lines."""

    __tablename__ = f"{prefix}cell_line"

    id: str = Field(default_factory=idg.cell_line, primary_key=True)
    ontology_id: str = Field(default=None, index=True, unique=True)
    name: str = Field(default=None, index=True)


@knowledge
class Pathway(SQLModel, table=True):  # type: ignore
    """Pathways."""

    id: str = Field(default_factory=idg.pathway, primary_key=True)
    ontology_id: str = Field(default=None, index=True, unique=True)
    name: str = Field(default=None, index=True)


@knowledge
class Phenotype(SQLModel, table=True):  # type: ignore
    """Phenotypes."""

    id: str = Field(default_factory=idg.phenotype, primary_key=True)
    ontology_id: str = Field(default=None, index=True, unique=True)
    name: str = Field(default=None, index=True)


@knowledge
class Readout(SQLModel, table=True):  # type: ignore
    """Biological readouts."""

    id: str = Field(default_factory=idg.readout, primary_key=True)
    efo_id: Optional[str] = Field(default=None, unique=True, index=True)
    name: Optional[str] = None
    molecule: Optional[str] = None
    instrument: Optional[str] = None
    measurement: Optional[str] = None
    created_by: str = CreatedBy
    created_at: datetime = CreatedAt
    updated_at: Optional[datetime] = UpdatedAt

    files: File = Relationship(
        back_populates="readouts",
        sa_relationship_kwargs=dict(secondary=FileReadout.__table__),
    )


File.readouts = relationship(
    Readout,
    back_populates="files",
    secondary=FileReadout.__table__,
)
File.__sqlmodel_relationships__["readouts"] = None



import os
from pathlib import Path

import lamindb as ln
from rich import print


def _upload_ontology_artifacts(
    instance: str, lndb_user: str, lndb_password: str, python_version: str = "3.10"
) -> None:
    """Uploads all ontology file artifacts to the specified instance.

    Uses all files generated by nox that are found in the _dynamic folder.

    Args:
        instance: The lndb instance to upload the files to.
        lndb_user: The lndb username.
        lndb_password: The associated password of the lndb user.
        python_version: The Python version that ran nox. Defaults to '3.10'.
    """
    _DYNAMIC_PATH = Path(
        f"{os.getcwd()}/.nox/build-package-bionty/lib/python{python_version}/site-packages/bionty/_dynamic"
    )

    ln.setup.login(lndb_user, password=lndb_password)
    ln.setup.load(instance, migrate=True)
    with ln.Session() as ss:
        transform = ln.select(
            ln.Transform, name="Bionty ontology artifacts upload"
        ).one_or_none()

        if transform is None:
            transform = ln.add(ln.Transform(name="Bionty ontology artifacts"))

        run = ln.Run(transform=transform)

        for filename in os.listdir(_DYNAMIC_PATH.absolute()):
            nox_ontology_file_path = os.path.join(_DYNAMIC_PATH.absolute(), filename)

            ontology_ln_file = ss.select(ln.File, name=filename).one_or_none()

            if ontology_ln_file is not None:
                print(
                    "[bold yellow]Found"
                    f" {ontology_ln_file.name}{ontology_ln_file.suffix} on S3."
                    " Skipping ingestion..."
                )
            else:
                ontology_ln_file = ln.File(
                    nox_ontology_file_path, key=filename, run=run
                )
                ss.add(ontology_ln_file)


_upload_ontology_artifacts(
    instance="sunnyosun/bionty-assets",
    lndb_user="testuser2@lamin.ai",
    lndb_password="goeoNJKE61ygbz1vhaCVynGERaRrlviPBVQsjkhz",
    python_version="3.9",
)



"""Development tools.

Tracking versions & migrations:

.. autosummary::
   :toctree: .

   version_zdno
   migration_zdno

Version of knowledge tables

.. autosummary::
   :toctree: .

   BiontyVersions
   CurrentBiontyVersions

Auxiliary modules:

.. autosummary::
   :toctree: .

   id
"""
from . import id
from ._bionty_versions import BiontyVersions, CurrentBiontyVersions
from ._versions import migration_zdno, version_zdno



import json
from typing import Literal, Optional

import pandas as pd

from .._entity import Entity
from ..dev._io import s3_bionty_assets
from ._shared_docstrings import _doc_params, doc_entites


@_doc_params(doc_entities=doc_entites)
class BFXPipeline(Entity):
    """Bioinformatics pipelines.

    Args:
        {doc_entities}
    """

    def __init__(
        self,
        species: str = "all",
        source: Optional[Literal["lamin"]] = None,
        version: Optional[str] = None,
        **kwargs
    ) -> None:
        super().__init__(
            source=source, version=version, species=species, reference_id="id", **kwargs
        )

    def df(self) -> pd.DataFrame:
        """DataFrame."""
        localpath = s3_bionty_assets(filename="bfxpipelines.json")
        with open(localpath, "r") as f:
            data = json.load(f)

        df = pd.DataFrame(data).transpose()

        return df



import pandas as pd

import bionty as bt


def test_pw_pathway_curation_ontology_id():
    df = pd.DataFrame(
        index=[
            "PW:0000049",
            "PW:0000009",
            "PW:0000001",
            "PW:0002000",
            "This pathway does not exist",
        ]
    )
    curated_df = bt.Pathway(database="pw", version="7.74").curate(df)

    curation = curated_df["__curated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert curation.equals(expected_series)


def test_pw_pathway_curation_name():
    df = pd.DataFrame(
        index=[
            "Toll-like receptor 9 signaling pathway",
            "Toll-like receptor TLR1:TLR2 signaling pathway",
            "classic metabolic pathway",
            "regulatory pathway",
            "This pathway does not exist",
        ]
    )
    curated_df = bt.Pathway(database="pw", version="7.74").curate(
        df, reference_id="name"
    )

    curation = curated_df["__curated__"].reset_index(drop=True)
    expected_series = pd.Series([True, True, True, True, False])

    assert curation.equals(expected_series)
